@@ -41,7 +41,7 @@ pyg_flags_compare(PyGFlags *self, PyObject *other)
 
     if (GET_INT_VALUE(self) == GET_INT_VALUE(other))
       return 0;
-    else if (GET_INT_VALUE(self) > GET_INT_VALUE(other))
+    else if (GET_INT_VALUE(self) < GET_INT_VALUE(other))
       return -1;
     else
       return 1;
@@ -80,8 +80,7 @@ pyg_flags_repr(PyGFlags *self)
     char *tmp, *retval;
     PyObject *pyretval;
   
-    tmp = generate_repr(self->gtype,
-			   self->parent.ob_ival);
+    tmp = generate_repr(self->gtype, self->parent.ob_ival);
 
     retval = g_strdup_printf("<flags %s of type %s>", tmp,
 			     g_type_name(self->gtype));
@@ -156,15 +155,19 @@ pyg_flags_from_gtype (GType gtype, int value)
     g_return_val_if_fail(gtype != G_TYPE_INVALID, NULL);
     
     pyclass = (PyObject*)g_type_get_qdata(gtype, pygflags_class_key);
-    if (pyclass == NULL)
-	return PyInt_FromLong(value);
+    if (pyclass == NULL) {
+	pyclass = pyg_flags_add(NULL, g_type_name(gtype), NULL, gtype);
+	if (!pyclass)
+	    return PyInt_FromLong(value);
+    }
 
+    
     values = PyDict_GetItemString(((PyTypeObject *)pyclass)->tp_dict,
 				  "__flags_values__");
     retval = PyDict_GetItem(values, PyInt_FromLong(value));
     if (!retval) {
 	PyErr_Clear();
-	
+
 	retval = ((PyTypeObject *)pyclass)->tp_alloc((PyTypeObject *)pyclass, 0);
 	g_assert(retval != NULL);
 	
@@ -182,6 +185,7 @@ pyg_flags_add (PyObject *   module,
 	       const char * strip_prefix,
 	       GType        gtype)
 {
+    PyGILState_STATE state;
     PyObject *instance_dict, *stub, *values;
     GFlagsClass *eclass;
     int i;
@@ -190,6 +194,8 @@ pyg_flags_add (PyObject *   module,
     g_return_val_if_fail(typename != NULL, NULL);
     g_return_val_if_fail(g_type_is_a(gtype, G_TYPE_FLAGS), NULL);
     
+    state = pyg_gil_state_ensure();
+
     instance_dict = PyDict_New();
     stub = PyObject_CallFunction((PyObject *)&PyType_Type, "s(O)O",
                                  typename, (PyObject *)&PyGFlags_Type,
@@ -197,7 +203,7 @@ pyg_flags_add (PyObject *   module,
     Py_DECREF(instance_dict);
     if (!stub) {
 	PyErr_SetString(PyExc_RuntimeError, "can't create const");
-	return NULL;
+	pyg_gil_state_release(state);
     }
     
     PyDict_SetItemString(((PyTypeObject *)stub)->tp_dict,
@@ -231,7 +237,7 @@ pyg_flags_add (PyObject *   module,
 						   strip_prefix),
 			 item);
       Py_INCREF(item);
-  }
+    }
     
     PyDict_SetItemString(((PyTypeObject *)stub)->tp_dict,
 			 "__flags_values__", values);
@@ -239,6 +245,8 @@ pyg_flags_add (PyObject *   module,
 
     g_type_class_unref(eclass);
 
+    pyg_gil_state_release(state);
+
     return stub;
 }
 
